<?php declare(strict_types=1);

namespace MyPrayerJournal;

use DateTimeImmutable;
use DateTimeZone;

/**
 * User interface building blocks
 */
class UI
{
    /**
     * Generate a material icon
     *
     * @param string $name The name of the material icon
     * @return string The material icon wrapped in a `span` tag
     */
    public static function icon(string $name): string {
        return "<span class=material-icons>$name</span>";
    }

    /**
     * Create a card when there are no results found
     */
    public static function noResults(string $heading, string $link, string $buttonText, string $text): void
    { ?>
        <div class=card>
        <h5 class=card-header><?=$heading?></h5>
        <div class="card-body text-center">
            <p class=card-text><?=$text?></p><?php
            self::pageLink($link, $buttonText, ['class' => 'btn btn-primary']); ?>
        </div>
        </div><?php
    }

    /**
     * Generate a link to a page within myPrayerJournal
     *
     * @param string $href The URL for the link
     * @param string $text The text for the link
     * @param array $attrs Any additional attributes that should be placed on the `a` tag
     */
    public static function pageLink(string $href, string $text, array $attrs = []): void
    { ?>
        <a href="<?=$href?>" hx-get="<?=$href?>" hx-target=#top hx-swap=innerHTML hx-push-url=true<?php
        foreach ($attrs as $key => $value) echo " $key=\"" . htmlspecialchars($value) . "\""; ?>><?=$text?></a><?php
    }

    // Thanks, William Entriken!  https://stackoverflow.com/a/42446994/276707
    /** @var array Periods for the relative date/time display */
    private static array $periods = [
        [           60,           1, '%s seconds ago', 'a second ago'],
        [       60*100,          60, '%s minutes ago', 'one minute ago'],
        [      3600*70,        3600, '%s hours ago',   'an hour ago'],
        [   3600*24*10,     3600*24, '%s days ago',    'yesterday'],
        [   3600*24*30,   3600*24*7, '%s weeks ago',   'a week ago'],
        [3600*24*30*30,  3600*24*30, '%s months ago',  'last month'],
        [          INF, 3600*24*265, '%s years ago',   'last year']
    ];

    /**
     * @throws \Exception
     */
    public static function relativeDate(string $date): void
    {
        $parsed = new DateTimeImmutable($date);
        $diff   = time() - $parsed->getTimestamp();
        foreach (self::$periods as $period) {
            if ($diff > $period[0]) continue;
            $diff = floor($diff / $period[1]);
            $value = $diff > 1 ? sprintf($period[2], $diff) : $period[3];
            $inZone = $parsed->setTimezone(new DateTimeZone($_SERVER['HTTP_X_TIME_ZONE'] ?? 'Etc/UTC'));
            echo '<span title="' . date_format($inZone, 'l, F j, Y \a\t g:ia T') . '">' . $value . '</span>';
            break;
        }
    }

}
